"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDtsExports = void 0;
var source_map_js_1 = require("source-map-js");
var classTransforms_1 = require("./classTransforms");
var validVarRegexp_1 = require("./validVarRegexp");
var isValidVariable = function (className) {
    return validVarRegexp_1.VALID_VARIABLE_REGEXP.test(className);
};
var classNameToProperty = function (className) { return "'".concat(className, "': string;"); };
var classNameToNamedExport = function (className) {
    return "export let ".concat(className, ": string;");
};
var flattenClassNames = function (previousValue, currentValue) {
    if (previousValue === void 0) { previousValue = []; }
    return previousValue.concat(currentValue);
};
var createDtsExports = function (_a) {
    var _b;
    var cssExports = _a.cssExports, fileName = _a.fileName, logger = _a.logger, options = _a.options;
    var classes = cssExports.classes;
    var processedClasses = Object.keys(classes)
        .map((0, classTransforms_1.transformClasses)(options.classnameTransform))
        .reduce(flattenClassNames, []);
    var filteredClasses = processedClasses
        .filter(isValidVariable)
        .map(classNameToNamedExport);
    var dts = "declare let classes: {\n".concat(processedClasses.map(classNameToProperty).join('\n  '), "\n};\nexport default classes;\n");
    if (options.namedExports !== false && filteredClasses.length) {
        dts += filteredClasses.join('\n') + '\n';
    }
    if (options.goToDefinition) {
        if (!cssExports.sourceMap)
            return dts;
        // Create a new source map consumer.
        var sourceMapConsumer_1 = new source_map_js_1.SourceMapConsumer(cssExports.sourceMap);
        // Split original CSS file into lines.
        var cssLines_1 = ((_b = cssExports.css) === null || _b === void 0 ? void 0 : _b.split('\n')) || [];
        // Create new equal size array of empty strings.
        var dtsLines_1 = Array.from(Array(cssLines_1.length), function () { return []; });
        var processedClasses_1 = Object.entries(classes)
            .map(function (_a) {
            var className = _a[0], hashedClassName = _a[1];
            return (0, classTransforms_1.transformClasses)(options.classnameTransform)(className).map(function (className) { return [className, hashedClassName]; });
        })
            .reduce(function (prev, curr) { return prev.concat(curr); }, []);
        var filteredClasses_1 = processedClasses_1.filter(function (_a) {
            var className = _a[0];
            return isValidVariable(className);
        });
        filteredClasses_1.forEach(function (_a) {
            var className = _a[0], hashedClassName = _a[1];
            var matchedLine = cssLines_1.findIndex(function (line) {
                return line.includes(hashedClassName);
            });
            var matchedColumn = matchedLine && cssLines_1[matchedLine].indexOf(hashedClassName);
            var lineNumber = sourceMapConsumer_1.originalPositionFor({
                // Lines start at 1, not 0.
                line: matchedLine >= 0 ? matchedLine + 1 : 1,
                column: matchedColumn >= 0 ? matchedColumn : 0,
            }).line;
            var index = lineNumber ? lineNumber - 1 : 0;
            var prev = dtsLines_1[index];
            dtsLines_1[index] = prev == null ? [className] : __spreadArray(__spreadArray([], prev, true), [className], false);
        });
        dts = dtsLines_1
            .map(function (classNames) {
            return classNames.reduce(function (prev, className) {
                return "".concat(prev).concat(classNameToNamedExport(className));
            }, '');
        })
            .join('\n');
    }
    if (options.customTemplate) {
        // eslint-disable-next-line @typescript-eslint/no-var-requires
        var customTemplate = require(options.customTemplate);
        return customTemplate(dts, {
            classes: classes,
            fileName: fileName,
            logger: logger,
        });
    }
    return dts;
};
exports.createDtsExports = createDtsExports;
