"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCssExports = exports.getFileType = void 0;
var path_1 = __importDefault(require("path"));
var sass_1 = __importDefault(require("sass"));
var icss_utils_1 = require("icss-utils");
var tsconfig_paths_1 = require("tsconfig-paths");
var sassTildeImporter_1 = require("../importers/sassTildeImporter");
var url_1 = require("url");
var getFileType = function (fileName) {
    if (fileName.endsWith('.css'))
        return "css" /* FileType.css */;
    if (fileName.endsWith('.sass'))
        return "sass" /* FileType.sass */;
    return "scss" /* FileType.scss */;
};
exports.getFileType = getFileType;
var getFilePath = function (fileName) { return path_1.default.dirname(fileName); };
var getCssExports = function (_a) {
    var css = _a.css, fileName = _a.fileName, logger = _a.logger, options = _a.options, processor = _a.processor, compilerOptions = _a.compilerOptions;
    try {
        var fileType = (0, exports.getFileType)(fileName);
        var rendererOptions = options.rendererOptions || {};
        var transformedCss = '';
        var sourceMap = void 0;
        if (options.customRenderer) {
            // eslint-disable-next-line @typescript-eslint/no-var-requires
            var customRenderer = require(options.customRenderer);
            transformedCss = customRenderer(css, {
                fileName: fileName,
                logger: logger,
                compilerOptions: compilerOptions,
            });
        }
        else if (fileType === "sass" /* FileType.sass */ || fileType === "scss" /* FileType.scss */) {
            var filePath = getFilePath(fileName);
            var _b = rendererOptions.sass || {}, loadPaths = _b.loadPaths, sassOptions = __rest(_b, ["loadPaths"]);
            var baseUrl = compilerOptions.baseUrl, paths = compilerOptions.paths;
            var matchPath_1 = baseUrl && paths ? (0, tsconfig_paths_1.createMatchPath)(path_1.default.resolve(baseUrl), paths) : null;
            var aliasImporter = {
                findFileUrl: function (url) {
                    var newUrl = matchPath_1 !== null
                        ? matchPath_1(url, undefined, undefined, [
                            ".".concat("sass" /* FileType.sass */),
                            ".".concat("scss" /* FileType.scss */),
                        ])
                        : undefined;
                    return newUrl ? (0, url_1.pathToFileURL)(newUrl) : null;
                },
            };
            var importers = [aliasImporter, sassTildeImporter_1.sassTildeImporter];
            var result = sass_1.default.compile(fileName, __assign({ loadPaths: __spreadArray([filePath, 'node_modules'], (loadPaths || []), true), importers: importers, sourceMap: true }, sassOptions));
            transformedCss = result.css.toString();
            sourceMap = JSON.stringify(result.sourceMap);
        }
        else {
            transformedCss = css;
        }
        var processedCss = processor.process(transformedCss, {
            from: fileName,
            map: {
                inline: false,
                prev: sourceMap,
            },
        });
        return {
            classes: processedCss.root
                ? (0, icss_utils_1.extractICSS)(processedCss.root).icssExports
                : {},
            css: processedCss.css,
            sourceMap: processedCss.map.toJSON(),
        };
    }
    catch (e) {
        logger.error(e);
        return { classes: {} };
    }
};
exports.getCssExports = getCssExports;
